package com.nisum.offertransactionservice.dao;

import com.nisum.offertransactionservice.config.OfferTransactionConfig;
import com.nisum.offertransactionservice.model.Item;
import com.nisum.offertransactionservice.model.OfferLookup;
import com.nisum.offertransactionservice.model.PERequest;
import com.nisum.offertransactionservice.model.PEResponse;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;
@RunWith(MockitoJUnitRunner.class)
@PropertySource("classpath:application.properties")
public class OfferDaoTest {

    @Mock
    private RestTemplate restTemplate;

    @InjectMocks
    private OfferDao offerDao = new OfferDao();

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Value("${promotion.engine.calculate.discount.url}")
    private String promotionEngineUrl ;

    @Test
    public void test(){

        Mockito.when(restTemplate.postForEntity("http://localhost:8081"+promotionEngineUrl,getPeRequest(), PEResponse.class))
                .thenReturn(new ResponseEntity<PEResponse>(getPeResponse(), HttpStatus.OK));
        PEResponse peResponse = offerDao.callPE(getPeRequest());
        Assert.assertEquals(peResponse,getPeResponse());
    }

    private PERequest getPeRequest(){
        PERequest peRequest = new PERequest();
        peRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke","12",13.05,"cmsCoupons");
        Item item2 = new Item("Thumpsup","11",14.05,"cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        peRequest.setItemList(itemList);
        OfferLookup offerLookup1 = new OfferLookup("1","12","234","02","AND","BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2","13","234","02","OR","BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        peRequest.setEligibleOffers(offerLookups);
        return peRequest;
    }

    private PEResponse getPeResponse(){
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke","12",6.05,"cmsCoupons");
        Item item2 = new Item("Thumpsup","11",12.32,"cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        peResponse.setDiscountedItemList(itemList);
        return peResponse;
    }
}
