package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.converter.OfferConverter;
import com.nisum.offertransactionservice.converter.PEResponseToOfferTransactionResConverter;
import com.nisum.offertransactionservice.dao.OfferDao;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.model.*;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@RunWith(MockitoJUnitRunner.class)
public class OfferServiceTest {
    @Mock
    private OfferConverter offerConverter;

    @Mock
    private OfferLookupRepo offerLookupRepo;

    @Mock
    private OfferDao offerDao;
    @Mock
    private PEResponseToOfferTransactionResConverter peResponseToOfferTransactionResConverter;

    @InjectMocks
    private OfferCallingPEService offerCallingPEService = new OfferCallingPEService();

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void serviceTest() {
        OfferTransactionRequest offerTransactionRequest = getOfferTransactionRequest();
        Mockito.when(offerLookupRepo.findAll()).thenReturn(returnOffers());
        Mockito.when(offerConverter.apply(getOfferTransactionRequest())).thenReturn(getPEReq());
        Mockito.when(offerDao.callPE(ArgumentMatchers.any())).thenReturn(getPeResponse());
        Mockito.when(peResponseToOfferTransactionResConverter.apply(getPeResponse())).thenReturn(retOfferTransRes());

       OfferTransactionResponse offerTransactionResponse =  offerCallingPEService.getDiscountedItemList(offerTransactionRequest);
        Assert.assertEquals(offerTransactionResponse,retOfferTransRes());
    }

    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke","12",6.05,"cmsCoupons");
        Item item2 = new Item("Thumpsup","11",12.32,"cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        return offerTransactionResponse;
    }

    private PEResponse getPeResponse(){
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke","12",6.05,"cmsCoupons");
        Item item2 = new Item("Thumpsup","11",12.32,"cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        peResponse.setDiscountedItemList(itemList);
        return peResponse;
    }

    private PERequest getPEReq() {
        PERequest peRequest = new PERequest();
        peRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke","12",13.05,"cmsCoupons");
        Item item2 = new Item("Thumpsup","11",14.05,"cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        peRequest.setItemList(itemList);
        /*OfferLookup offerLookup1 = new OfferLookup("1","12","234","02","AND","BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2","13","234","02","OR","BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        peRequest.setEligibleOffers(offerLookups);*/
        return peRequest;
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05, "cmsCoupons");
        Item item2 = new Item("Thumpsup", "11", 14.05, "cmsCoupons");
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionRequest.setItemList(itemList);
        return offerTransactionRequest;
    }

    public Iterable<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", "234", "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", "234", "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }
}
